<?php
/**
* @version $Id: template.php 5256 2006-10-01 22:13:10Z webImagery $
* @package Joomla
* @copyright Copyright (C) 2005 - 2006 Open Source Matters. All rights reserved.
* @license GNU/GPL, see LICENSE.php
* Joomla! is free software. This version may have been modified pursuant
* to the GNU General Public License, and as distributed it includes or
* is derivative of works licensed under the GNU General Public License or
* other free or open source software licenses.
* See COPYRIGHT.php for copyright notices and details.
*/

/**
 * Template installer
 *
 * @package		Joomla.Framework
 * @subpackage	Installer
 * @since		1.5
 */
class JInstallerTemplate extends JInstaller
{

	/**
	 * Custom install method
	 *
	 * @access	public
	 * @param	string	$p_fromdir	Directory from which to install the template
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function install($p_fromdir)
	{

		/*
		 * First lets set the installation directory, find and check the installation file and verify
		 * that it is the proper installation type
		 */
		if (!$this->preInstallCheck($p_fromdir, 'template'))
		{
			return false;
		}

		// Get the root node of the XML document
		$root = & $this->_xmldoc->documentElement;

		/*
		 * LEGACY CHECK
		 */
		$version	= $root->getAttribute('version');
		$rootName	= $root->getTagName();
		$config		= &JFactory::getConfig();
		if ((version_compare($version, '1.5', '<') || $rootName == 'mosinstall') && !$config->getValue('config.legacy')) {
			JError::raiseWarning(1, JText::_('MUSTENABLELEGACY'));
			return false;
		}

		/*
		 * Get the client application target
		 */
		if ($client = $root->getAttribute('client'))
		{
			$clientVals = JApplicationHelper::getClientInfo($client, true);
			if ($clientVals === false)
			{
				JError::raiseWarning(1, 'JInstallerTemplate::install: '.JText::_('Unknown client type').' ['.$client.']');
				return false;
			}
			$basePath = $clientVals->path;
			$clientId = $clientVals->id;
		} else
		{
			/*
			 * No client attribute was found so we assume the site as the client
			 */
			$client = 'site';
			$basePath = JPATH_SITE;
			$clientId = 0;
		}

		/*
		 * Set the template name
		 */
		$e = & $root->getElementsByPath('name', 1);
		$this->_extensionName = $e->getText();

		/*
		 * Set the template installation directory
		 */
		$this->_extensionDir = JPath::clean($basePath.DS.'templates'.DS.strtolower(str_replace(" ", "_", $this->_extensionName)));

		/*
		 * If the template directory does not exists, lets create it
		 */
		$created = false;
		if (JFolder::exists($this->_extensionDir)) {
			JError::raiseWarning(1, 'JInstallerTemplate::install: '.JText::_('Template Already Exists'));
			return false;
		} else {
			if (!$created = JFolder::create($this->_extensionDir)) {
				JError::raiseWarning(1, 'JInstallerTemplate::install: '.JText::_('Failed to create directory').' "'.$this->_extensionDir.'"');
				return false;
			}
		}

		/*
		 * Since we created the template directory and will want to remove it if we have to roll back
		 * the installation, lets add it to the installation step stack
		 */
		$this->_stepStack[] = array ('type' => 'folder', 'path' => $this->_extensionDir);

		/*
		 * Copy all necessary files
		 */
		if ($this->_parseFiles('files') === false)
		{
			JError::raiseWarning(1, 'JInstallerTemplate::install: '.JText::_('Failed to create directory').' "'.$this->_extensionDir.'"');

			// Install failed, rollback any changes
			$this->_rollback();
			return false;
		}
		if ($this->_parseFiles('images') === false)
		{
			JError::raiseWarning(1, 'JInstallerTemplate::install: '.JText::_('Failed to create directory').' "'.$this->_extensionDir.'"');

			// Install failed, rollback any changes
			$this->_rollback();
			return false;
		}
		if ($this->_parseFiles('css') === false)
		{
			JError::raiseWarning(1, 'JInstallerTemplate::install: '.JText::_('Failed to create directory').' "'.$this->_extensionDir.'"');

			// Install failed, rollback any changes
			$this->_rollback();
			return false;
		}
		if ($this->_parseFiles('media') === false)
		{
			JError::raiseWarning(1, 'JInstallerTemplate::install: '.JText::_('Failed to create directory').' "'.$this->_extensionDir.'"');

			// Install failed, rollback any changes
			$this->_rollback();
			return false;
		}

		/*
		 * Get the template description
		 */
		$e = & $root->getElementsByPath('description', 1);
		if (!is_null($e))
		{
			$this->description = $this->_extensionName.'<p>'.$e->getText().'</p>';
		} else
		{
			$this->description = $this->_extensionName;
		}

		/*
		 * Now, lets create the necessary template positions
		 */
		$templatePositions = & $root->getElementsByPath('install/positions', 1);
		if (!is_null($templatePositions))
		{
			$positions = $templatePositions->childNodes;
			foreach ($positions as $position)
			{

				$id = $this->_createTemplatePosition($position->getText());

				if ($id)
				{
					/*
					 * Since we have created a template positions item, we add it to the installation step stack
					 * so that if we have to rollback the changes we can undo it.
					 */
					$step = $this->_stepStack[] = array ('type' => 'position', 'id' => $id);
				}
			}
		}

		/*
		 * Lastly, we will copy the setup file to its appropriate place.
		 */
		if (!$this->_copyInstallFile(0))
		{
			JError::raiseWarning(1, 'JInstallerTemplate::install: '.JText::_('Could not copy setup file'));

			// Install failed, rollback changes
			$this->_rollback();
			return false;
		}
		return true;
	}

	/**
	 * Custom uninstall method
	 *
	 * @access	public
	 * @param	int		$path	The path of the template to uninstall
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function uninstall($path )
	{

		// Initialize variables
		$retval = true;

		/*
		 * For a template the id will be the template name which represents the
		 * subfolder of the templates folder that the template resides in.
		 */
		$path = trim($path);
		if (!$path)
		{
			JError::raiseWarning('SOME_ERROR_CODE', 'JInstallerTemplate::uninstall: '.JText::_('Template id is empty, cannot uninstall files'));
			return false;
		}

		/*
		 * Set some internal paths for smooth operation :)
		 */
		$this->_installDir = $path;
		$this->_extensionDir = $path;

		/*
		 * See if there is an xml install file
		 */
		if (!$this->_findInstallFile())
		{
			JError::raiseWarning('SOME_ERROR_CODE', 'JInstallerTemplate::uninstall: '.JText::_('Could not find xml install file'));
			return false;
		}

		/*
		 * Remove any files added to the Joomla images/ folder
		 */
		if ($this->_removeFiles('media') === false)
		{
			JError::raiseWarning('SOME_ERROR_CODE', 'JInstallerTemplate::uninstall: '.JText::_('Unable to remove media files'));
			$retval = false;
		}

		/*
		 * Delete the template directory
		 */
		if (JFolder::exists($path))
		{
			$retval = JFolder::delete(JPath::clean($path));
		} else
		{
			JError::raiseWarning('SOME_ERROR_CODE', 'JInstallerTemplate::uninstall: '.JText::_('Directory does not exist, cannot remove files'));
			$retval = false;
		}

		return $retval;
	}

	/**
	 * Creates a new template position if it doesn't exist already
	 *
	 * @access	private
	 * @param	string	$position	Template position to create
	 * @return	mixed	Template position id (int) if a position was inserted or
	 * boolean false otherwise
	 * @since	1.5
	 */
	function _createTemplatePosition($position)
	{

		/*
		 * Get the global database connector object
		 */
		$db = $this->_db;

		// Initialize variable
		$retval = false;

		if ($position)
		{
			$db->setQuery("SELECT id " .
					"\nFROM #__template_positions " .
					"\nWHERE position = '$position'");
			$db->Query();
			if (!$db->getNumRows())
			{
				$db->setQuery("INSERT INTO #__template_positions " .
						"\nVALUES (0,'$position','')");
				if ($db->Query() !== false)
				{
					$retval = $db->insertid();
				}
			}
		}
		return $retval;
	}

	/**
	 * Custom rollback method
	 * 	- Roll back the template position item
	 *
	 * @access	private
	 * @param	array	$arg	Installation step to rollback
	 * @return	boolean	True on success
	 * @since	1.5
	 */
	function _rollback_position($arg)
	{

		// Get database connector object
		$db = & $this->_db;

		/*
		 * Remove the entry from the #__template_positions table
		 */
		$query = "DELETE " .
				"\nFROM `#__template_positions` " .
				"\nWHERE id='".$arg['id']."'";

		$db->setQuery($query);

		return ($db->query() !== false);
	}
}
?>